classdef FlexWage < handle
    %FLEXWAGE Equilibrium of the model under flexible wages
    %   The flex wage equilbrium outcome is the one that occurs when
    %   wages are flexible and firms have market power in their
    %   labor market segment and goods market.
    %
    %   Properties
    %   ----------
    %   Model       Model to which this object is linked
    %   L           Employment / output
    %   u           unemployment rate
    %   Pi          inflation
    %   a           trend productivity growth rate
    %   r           real interest rate
    %   i           nominal interest rate
    %   V           value of wage setting problem
    %   h0          Spike at zero nominal wage change
    %   h04Q        Spike at zero for 4 quarter wage changes
    %
    %   Methods
    %   -------
    %   G           Distribution function of detrended real wages
    %   g           Density function of detrended real wages
    %
    properties
        
    end
    
    properties ( SetAccess = private , Hidden = false )
    
        Model
        
    end
    
    properties (Dependent = true, SetAccess = private)
        L
        u
        Pi
        a
        r
        i
        V
        h0
        h04Q
    end
    
    methods
        
        function obj = FlexWage( Model_ )
            % Constructor that links object to parent model
            obj.Model = Model_ ; 
        end % FlexWage
        
        function L_ = get.L(obj)
            Gam = obj.Model.Parameters.Gamma;
            eta = obj.Model.Parameters.Eta;
            Ztilde = obj.Model.Parameters.Ztilde;
            
            L_ = ( ((eta-1)./eta).*(1 / Ztilde) ).^(Gam./(1+Gam));
            
        end % Output get method
             
        function u_ = get.u(obj)
            
            u_ = 0;
            
        end % Unemployment rate get method
        
        function Pi_ = get.Pi(obj)
            
            Pi_ = obj.Model.Parameters.PiBar;
            
        end % Inflation rate get method
        
        function a_ = get.a(obj)
            
            a_ = obj.Model.Parameters.aBar;
            
        end % Trend productivity growth rate get method
        
        function r_ = get.r(obj)
            
            beta = obj.Model.Parameters.Beta;
            
            r_ = (1/beta)-1;
            
        end % Real interest rate rate get method
        
        function i_ = get.i(obj)
                                 
            i_ = (1+obj.r)*(1+obj.Pi)-1;
            
        end % Nominal interest rate rate get method        
        
        function h0_ = get.h0(obj)
                                 
            h0_ = 0;
            
        end % Nominal interest rate rate get method
        
        function h04Q_ = get.h04Q(obj)
            
            h04Q_ = 0;
            
        end % Spike at zero nominal wage changes of 4 quarters
           
        function V_ = get.V(obj)
           
            gam = obj.Model.Parameters.Gamma;
            eta = obj.Model.Parameters.Eta;
            r   = (eta-1).*gam./(gam+eta);
            Beta = obj.Model.Parameters.Beta;
            
            EZr = PolyFZInt( 1 , -r , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );
                 
            Markup = ((eta-1)./eta).^(r)-((eta-1)./eta).^(r+1);            
                                  
            V_ = (1./(1-Beta)).*EZr.*Markup.*( (1./obj.L).^(r.*(gam+1)./gam) );
            
        end
        
        function Gw = G(obj,w)

            gam = obj.Model.Parameters.Gamma;
            eta = obj.Model.Parameters.Eta;
            sigma = obj.Model.Parameters.Sigma;
            
            wlte0 = ( w <= 0 );
            wbt0 = ( w > 0 );
            wadj = w.*wbt0 + wlte0; 
            muw = -0.5*gam*(sigma^2)/(gam+eta) ...
                  + ((gam+1)./(eta+gam)).*log(obj.L) ...
                  + ( gam ./ (gam+eta) ).*log(eta./(eta-1)); 
            sigmaw = gam.*sigma/(gam+eta);    
                
            wUp = ( (obj.Model.Numerics.ZUp).^(gam./(gam+eta)) ).* ...
                    ( (obj.L).^((gam+1)./(gam+eta)) );
            wLow= ( (obj.Model.Numerics.ZLow).^(gam./(gam+eta)) ).* ...
                  ( (obj.L).^((gam+1)./(gam+eta)) );
            
            Gw = wbt0.*( normcdf( log(wadj) , muw , sigmaw ) - ...
                         normcdf( log(wLow) , muw , sigmaw ) )./ ...
                       ( normcdf( log(wUp) , muw , sigmaw ) - ...
                         normcdf( log(wLow) , muw , sigmaw ) ) ;
            
        end % Distribution of wages method
        
        function gw = g(obj,w)

            gam = obj.Model.Parameters.Gamma;
            eta = obj.Model.Parameters.Eta;
            sigma = obj.Model.Parameters.Sigma;
            
            wlte0 = ( w <= 0 );
            wbt0 = ( w > 0 );
            wadj = w.*wbt0 + wlte0; 
            muw = -0.5*gam*(sigma^2)/(gam+eta) ...
                  + ((gam+1)./(eta+gam)).*log(obj.L) ...
                  + ( gam ./ (gam+eta) ).*log(eta./(eta-1)); 
            sigmaw = gam.*sigma/(gam+eta);    
                
            wUp = ( (obj.Model.Numerics.ZUp).^(gam./(gam+eta)) ).* ...
                    ( (obj.L).^((gam+1)./(gam+eta)) );
            wLow= ( (obj.Model.Numerics.ZLow).^(gam./(gam+eta)) ).* ...
                  ( (obj.L).^((gam+1)./(gam+eta)) );    
                    
            gw = wbt0.*(1./(wbt0+wlte0)).*normpdf( log(wadj) , muw , sigmaw );
            gw = (w>wLow).*(w<wUp).*gw./ ...
                       ( normcdf( log(wUp) , muw , sigmaw ) - ...
                         normcdf( log(wLow) , muw , sigmaw ) ) ;
            
        end % Density of wages method   
            
    end
    
end

