classdef Numerics < handle
    %NUMERICS Numerical approximation parameters for DNWR and SRAS
    %   
    %
    %   Properties
    %   ----------
    %   Parent      Model to which this object is linked
    %   PUp         Excess probability for grid boundary
    %   PLow        Probability for gred boundary
    %   Dw          Degree of polynomial approximation of real wage
    %                   functions
    %   DZ          Degree of polynomial approximation of productivity
    %                   shocks
    %   wGrid       [ (Dw+1) x 1 ] - vector with real wage grid
    %   wUp         Maximum level of real wage in grid
    %   wLow        Minimum level of real wage in grid
    %   ZGrid       [ (DZ+1) x 1 ] - vector with
    %   ZUp         Maximum level of productivity shock in grid
    %   ZLow        Minimum level of productivity shock in grid
    %   T           Length of path for extended path method
    %   Tol         Tolerance for stopcriteria
    %   TPTol       Tolerance for stopcriteria of transition path
    %   MaxIt       Maximum Number of Iterations
    %   PolyOrder   Order of the polynomial approximation
    %   wPoints     Number of points on the wage grid
    %
    properties
        
        PUp = 0.0001;
        PLow = 0.0001;
        Dw = 25;
        DZ = 25;
        T = 75;
        Tol = 0.0001;
        TPTol = 0.2;
        MaxIt = 10000;
        PolyOrder = 6;
        wPoints = 40;
        
    end
    
    properties ( SetAccess = private , Hidden = false )
    
        Parent
        Type = 'numerics'
        
    end
    
    properties (Dependent = true, SetAccess = private)
        wGrid
        ZGrid
        wUp
        wLow
        ZUp
        ZLow
    end
    
    methods
        
        function obj = Numerics( MyParent )
            % Constructor that links object to parent model
            obj.Parent = MyParent;
        end % FlexWage
        
        % Methods used to change the parameter values
        function obj = set.PUp(obj,val)
           if ~(  ( val > 0 ) & ( val < (1-obj.PLow) ) ) 
              error('Pup not feasible');
           end
           obj.PUp = val;
        end
        
        function obj = set.PLow(obj,val)
           if ~(  ( val > 0 ) & ( val < (1-obj.PUp) ) ) 
              error('PLow not feasible');
           end
           obj.PLow = val;
        end
        
        function obj = set.Dw(obj,val)
           if ~(  ( val > 0 ) & isinteger(val) ) 
              error('Dw not feasible');
           end
           obj.Dw = val;
        end
        
        function obj = set.DZ(obj,val)
           if ~(  ( val > 0 ) & isinteger(val) ) 
              error('DZ not feasible');
           end
           obj.DZ = val;
        end
        
        function obj = set.T(obj,val)
           if ~(  ( val > 0 ) ) 
              error('T not feasible');
           end
           obj.T = val;
        end
        
        function obj = set.Tol(obj,val)
           if ~( val > 0 )
              error('Tol not feasible');
           end
           obj.Tol = val;
        end    
            
        function wGrid_ = get.wGrid(obj)
        
            StepSize = (obj.wUp-obj.wLow)/obj.Dw;
            wGrid_ = obj.wLow + StepSize.*(cumsum(ones(obj.Dw+1,1))-1);
            
        end
       
        function ZGrid_ = get.ZGrid(obj)
        
            StepSize = (obj.ZUp-obj.ZLow)/obj.DZ;
            ZGrid_ = obj.ZLow + StepSize.*(cumsum(ones(obj.DZ+1,1))-1);
            
        end
        
        function wUp_ = get.wUp(obj)
        
            gam = obj.Parent.Parameters.Gamma;
            theta = obj.Parent.Parameters.Theta;
            ksi = obj.Parent.Parameters.Ksi;
            eta = obj.Parent.Parameters.Eta;
            sigma = obj.Parent.Parameters.Sigma;
            
            muw = -0.5*(eta-1)*(sigma^2)/(eta+ksi) ...
                + (eta/(eta+ksi))*(log(ksi)-log(1+ksi)) ...
                + (eta/(eta+ksi))*(log(eta-1)-log(eta)) ...
                + (ksi/(eta+ksi))*log( obj.Parent.FlexWage.wtilde ) ...
                + (1/(eta+ksi))*log( obj.Parent.FlexWage.Y ) ...
                - (1/(eta+ksi))*log( obj.Parent.FlexWage.Ltilde ) ;
            sigmaw = (eta-1)*sigma/(eta+ksi);    
                    
            wUp_ = exp(norminv( 1-obj.PUp , muw , sigmaw ));           
            
        end
        
        function wLow_ = get.wLow(obj)
        
            gam = obj.Parent.Parameters.Gamma;
            theta = obj.Parent.Parameters.Theta;
            ksi = obj.Parent.Parameters.Ksi;
            eta = obj.Parent.Parameters.Eta;
            sigma = obj.Parent.Parameters.Sigma;
            
            muw = -0.5*(eta-1)*(sigma^2)/(eta+ksi) ...
                + (eta/(eta+ksi))*(log(ksi)-log(1+ksi)) ...
                + (eta/(eta+ksi))*(log(eta-1)-log(eta)) ...
                + (ksi/(eta+ksi))*log( obj.Parent.FlexWage.wtilde ) ...
                + (1/(eta+ksi))*log( obj.Parent.FlexWage.Y ) ...
                - (1/(eta+ksi))*log( obj.Parent.FlexWage.Ltilde ) ;
            sigmaw = (eta-1)*sigma/(eta+ksi);    
                    
            wLow_ = exp(norminv( obj.PLow , muw , sigmaw ));
            
        end
        
        function ZUp_ = get.ZUp(obj)
        
            sigma = obj.Parent.Parameters.Sigma;
            
            ZUp_ = exp(norminv( 1-obj.PUp, -0.5*(sigma^2) , sigma ));
            
        end
        
        function ZLow_ = get.ZLow(obj)
        
            sigma = obj.Parent.Parameters.Sigma;
            
            ZLow_ = exp(norminv( obj.PLow, -0.5*(sigma^2) , sigma ));
            
        end
        
    end
    
end

