classdef Parameters < handle
    %PARAMETERS parameters of the DNWR and SRAS model 
    %   Set and retrieve the parameters of the model
    %   
    %   1. Preference parameters
    %   ------------------------
    %   Eta       own wage elasticity of labor demand
    %   Beta      discount factor
    %   Gamma     "Frisch elasticity" of labor supply
    %
    %   2. Technology parameters
    %   ------------------------
    % 	Sigma     standard deviation of productivity shocks
    %   Lamda     probability of DNWR constraint
    %   aBar      trend productivity growth
    %
    %   3. Policy rule parameters
    %   -------------------------
    %   PiBar     target inflation rate
    %   PhiY      output coefficient in Taylor Rule
    %   PhiPi     inflation coefficient in Taylor Rule
    %
    %   4. Dependent properties
    %   -----------------------
    %   Ztilde    Aggregate productivity level in first best
    %
    
    properties
        
        % 1. Preference parameters
        Eta = 10;
        Beta = 0.9875;
        Gamma = 0.5;
        
        % 2. Technology parameters
        Sigma = 1;
        Lamda = 0.001;
        aBar = 0.0066;
        
        % 3. Policy rule parameters
        PiBar = 0.005;
        PhiY  = 0.25;
        PhiPi = 0.25;
        
    end
    
    properties ( SetAccess = private , Hidden = false )
    
        Model
        
    end
    
    properties (Dependent = true, SetAccess = private)
        Ztilde
    end
    
    methods
        
        function obj = Parameters( MyParent )
            % Constructor that tracks what type of parent object has
            if ~strcmp(MyParent.Type,'model')
                error('Parent object needs to be a model object');
            end                     
            
            obj.Model = MyParent;            
                       
        end
        
        % Methods used to change the parameter values
        function obj = set.Eta(obj,val)
           if ~( val > 1 ) 
              error('eta must be larger than one')
           end
           obj.Eta = val;
        end

        function obj = set.Beta(obj,val)
           if ~ ( ( val > 0 ) & ( val < 1 ) ) 
              error('discount factor needs to be between zero and one')
           end
           obj.Beta = val;
        end
        
        function obj = set.Sigma(obj,val)
           if ~( val > 0 ) 
              error('Standard deviation of productivity shocks needs to be positive')
           end 
           obj.Sigma = val;
        end
        
        function obj = set.Lamda(obj,val)
           if ~ ( ( val >= 0 ) & ( val < 1 ) ) 
              error('Probability of DNWR constraint must be greater or equal to 0 and smaller than 1')
           end
           obj.Lamda = val;
        end
        
        function obj = set.PiBar(obj,val)
           if ~ ( ( val < 1 ) & ( val > -1 ) ) 
              error('Are you sure you would like to set the target inflation rate to this value?')
           end
           obj.PiBar = val;
        end
        
        function obj = set.aBar(obj,val)
           if ~ ( ( val < 1 ) & ( val > -1 ) ) 
              error('Are you sure you would like to set the trend productivity growth rate to this value?')
           end
           obj.aBar = val;
        end
        
        function obj = set.PhiY(obj,val)
           if ~( val >= 0 ) 
              error('Output coefficient in Taylor rule must be non-negative')
           end 
           obj.PhiY = val;
        end
        
        function obj = set.PhiPi(obj,val)
           if ~( val >= 0 ) 
              error('Inflation coefficient in Taylor rule must be non-negative')
           end 
           obj.PhiPi = val;
        end              
        
        function ztilde = get.Ztilde(obj)
            r = -obj.Gamma.*(obj.Eta-1)/(obj.Gamma+obj.Eta);
            Mu = -1.*0.5.*(obj.Sigma^2);

            F_ZUp = normcdf( log(obj.Model.Numerics.ZUp) , Mu , obj.Sigma );
            F_ZLow = normcdf( log(obj.Model.Numerics.ZLow) , Mu , obj.Sigma );
            Rescale = F_ZUp - F_ZLow ;

            atilde = ( log(obj.Model.Numerics.ZLow)+ 0.5.*(obj.Sigma^2) )./obj.Sigma ;
            btilde = ( log(obj.Model.Numerics.ZUp)+ 0.5.*(obj.Sigma^2) )./obj.Sigma ;

            ztilde = exp(0.5.*(r-1).*(obj.Sigma^2)).* ... 
                    ((( normcdf( r.*obj.Sigma - atilde , 0 , 1 ) - normcdf( r.*obj.Sigma - btilde , 0 , 1 ))./Rescale ).^(1./r));
            
        end % Modulus get method
        
    end
    
end

