classdef SteadyState < handle
    %STEADYSTATE Steady state of the model
    %
    %   Properties
    %   ----------
    %   Type        Object type 'steady state'
    %   Model       Model for which object is steady state
    %   L           Level of output in the steady state
    %   ZStar       Adjusted disutility level.
    %   wMax        Maximum real wage charged
    %   wMin        Minimum real wage charged
    %   u           Level of the unemployment rate in the steady state
    %   Pi          Level of inflation in the steady state
    %   a           Level of productivity growth in the steady state
    %   r           Level of the real interest rate in the steady state
    %   i           Level of the nominal interest rate in the steady state
    %   w_Zp        Polynomial with the wage-setting function
    %   Z_wp        Polynomial with the inverse of the wage-setting
    %                   function
    %   V_wp        Polynomial with the value function approximation
    %   G_wp        Polynomial with the wage distribution approximation
    %   h0          Spike at zero nominal wage change
    %   h04Q        Spike at zero nominal wage change over 4 quarters
    %   IsSolved    Boolean that flags whether steady state values are 
    %               for the current model parameters or whether it needs
    %               to be solved for these parameters.   
    %
    %   Methods
    %   -------
    %   G           Distribution function of real wages
    %   g           Density function of real wages
    %   H           Distribution function of log nominal wage changes
    %   h           Density function of log nominal wage changes
    %   Solve       Solve for the steady state
    %   
    
    properties
    end
    
    properties ( SetAccess = private , Hidden = false )
        Type = 'steady state'
        Model
        L
        ZStar
        wMin
        wMax
        u        
        w_Zp
        Z_wp
        V_wp
        G_wp
    end
    
    properties ( SetAccess = private , Hidden = true )
        
        Eta_s
        Beta_s
        Gamma_s
        Sigma_s
        Lamda_s
        PiBar_s
        aBar_s        
        
    end
    
    properties ( Dependent = true , SetAccess = private )
       
        Pi
        a
        r
        i
        h0
        h04Q
        IsSolved
        
    end
    
    methods
        
        function obj = SteadyState( MyParent )
            % Constructor that tracks what type of parent object has
            if ~strcmp(MyParent.Type,'model')
                error('Parent object needs to be a model object');
            end                     
            
            obj.Model = MyParent;            
            
            obj.L = 0;
            obj.ZStar = 0;
            obj.wMax = 0;
            obj.wMin = 0;
            obj.u = 0;
            obj.w_Zp = 0;             
            obj.Z_wp = 0;
            obj.V_wp = 0;
            obj.G_wp = 0;  

            obj.Eta_s = 0;
            obj.Beta_s = 0;
            obj.Gamma_s = 0; 
            obj.Sigma_s = 0;
            obj.Lamda_s = 0;
            obj.PiBar_s = 0;
            obj.aBar_s = 0;
            
        end
        
        function Pi_ = get.Pi(obj)
        
            Pi_ = obj.Model.FirstBest.Pi ;
            
        end
        
        function a_ = get.a(obj)
        
            a_ = obj.Model.FirstBest.a;
            
        end
        
        function r_ = get.r(obj)
        
            r_ = obj.Model.FirstBest.r;
            
        end
        
        function i_ = get.i(obj)
        
            i_ = obj.Model.FirstBest.i;
            
        end
             
        function IsSolved_ = get.IsSolved(obj)
           
            IsSolved_ = ...
                ( obj.Eta_s == obj.Model.Parameters.Eta ) & ...
                ( obj.Beta_s == obj.Model.Parameters.Beta ) & ...
                ( obj.Gamma_s == obj.Model.Parameters.Gamma ) & ... 
                ( obj.Sigma_s == obj.Model.Parameters.Sigma ) & ...
                ( obj.Lamda_s == obj.Model.Parameters.Lamda ) & ...
                ( obj.aBar_s == obj.Model.Parameters.aBar ) & ...
                ( obj.PiBar_s == obj.Model.Parameters.PiBar );
            
        end     
        
        function h0_ = get.h0(obj)
        
            if obj.IsSolved
                    
                ZGrid = obj.Model.Numerics.ZGrid;
                w_Z = polyval( obj.w_Zp , ZGrid );
                G_wg = ppvalcutoff( obj.G_wp , w_Z.*(1+obj.Pi).*(1+obj.a) , 0 , 1 );                
                OneMinusG_wg = 1 - G_wg;
                OneMinusG_wp = polyfit( ZGrid , OneMinusG_wg  , obj.Model.Numerics.PolyOrder );
                h0_ = obj.Model.Parameters.Lamda .* ...
                      PolyFZInt( OneMinusG_wp , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );
                
            else    
                h0_ = NaN;
            end
            
        end
        
        function h04Q_ = get.h04Q(obj)
            
            h04Q_ = H04QCalculation( obj.G_wp , obj.Z_wp , obj.Pi , obj.a , obj.Model );
            
        end % Spike at zero nominal wage changes of 4 quarters
        
        function Solved_ = Solve( obj )      
            
            if ~( obj.IsSolved )
               
                Tol = obj.Model.Numerics.Tol;
                MaxIt = obj.Model.Numerics.MaxIt;
                                                
                % Start the iterations off in the flexible wage equilibrium
                L_prev = obj.Model.FlexWage.L;
                
                Gam = obj.Model.Parameters.Gamma;
                Eta = obj.Model.Parameters.Eta;                                
                
                wHatUp = (((Eta./(Eta-1)).*obj.Model.Numerics.ZUp).^(Gam./(Gam+Eta))).* ...
                            ( L_prev.^((Gam+1)./(Gam+Eta)) );                 
                wHatLow = (((Eta./(Eta-1)).*obj.Model.Numerics.ZLow).^(Gam./(Gam+Eta))).* ...
                            ( L_prev.^((Gam+1)./(Gam+Eta)) );
                GridPoints = obj.Model.Numerics.wPoints;
                wg = wHatLow +(cumsum(ones(GridPoints,1))-1).*(wHatUp-wHatLow)./(GridPoints-1);
                
                V_wp_prev = [ zeros( 1,  obj.Model.Numerics.PolyOrder ) 1 ].*obj.Model.FlexWage.V;
                G_wp_prev = obj.Model.FlexWage.G( wg );
                G_wp_prev = pchip( wg , G_wp_prev );
                               
                rt = obj.r;
                Pit = obj.Pi;
                at = obj.a;
                
                SSConverged = false;
                SSiter = 0;
                Criterion = NaN;
                
                while ( SSConverged == false )                                    
                        
                    msghead1 = strcat('Solving:  ', obj.Model.Name );
                    msghead2 = strcat('steady state iteration:  ',num2str(SSiter),', criterion:  ', num2str(Criterion) );
                    
                    VConverged = false;
                    Viter = 0;
                    while ( VConverged == false )                                                                  
                        
                        [ w_Zp_ , Z_wp_ , V_wp_ ] = ... 
                            ValueFunctionIteration( V_wp_prev , L_prev , 0 , Pit , at , obj.Model );
                        
                        Criterion = ( mean( abs(V_wp_ - V_wp_prev) ) );
                        VConverged = ( Criterion < Tol );
                        Viter = Viter + 1;
                        
                        msg = strcat('Value function iteration:  ',num2str(Viter),', criterion:  ', num2str(Criterion) );
                        if ( obj.Model.Display > 0 )
                            clc;
                		    disp(msghead1);
                            disp(msghead2);
                            disp(msg);
                        end
			    
                        if ( Viter > MaxIt )
                            error('Maximum number of iterations exceeded in value function recursion')
                        end
                        
                        V_wp_prev = V_wp_;
                        
                    end
                    
                    W_Low = polyval( w_Zp_ , obj.Model.Numerics.ZLow);
                    W_Up = polyval( w_Zp_ , obj.Model.Numerics.ZUp);
                    
                    wg = W_Low +(cumsum(ones(GridPoints,1))-1).*(W_Up - W_Low)./(GridPoints-1);
                    G_wp_prevg = ppvalcutoff( G_wp_prev , wg , 0 , 1 );
                    G_wp_prev = pchip( wg , G_wp_prevg );
                    
                    GConverged = false;
                    Giter = 0;
                    while ( GConverged == false )                  
                        
                        [ g_wp_ , G_wp_ ] = ... 
                            WageDistributionIteration( G_wp_prev , Z_wp_ , w_Zp_ , Pit, at , obj.Model );
                        
                        G_wp_g = ppvalcutoff( G_wp_ , [G_wp_.breaks' ; G_wp_prev.breaks'] , 0 , 1 );
                        G_wp_prevg = ppvalcutoff( G_wp_prev , [G_wp_.breaks' ; G_wp_prev.breaks'] , 0 , 1 );
                        
                        Criterion = mean( abs(G_wp_g - G_wp_prevg) );
                        GConverged = ( Criterion < Tol );
                        Giter = Giter + 1;
                        
                        msg = strcat('Wage distribution iteration:  ',num2str(Giter),', criterion:  ', num2str(Criterion) );
                        if ( obj.Model.Display > 0 )
                            clc;
                            disp(msghead1);
                            disp(msghead2);
                            disp(msg);
                        end                        
                        
                        if ( Giter > MaxIt )
                            error('Maximum number of iterations exceeded in value function recursion')
                        end
                        G_wp_prev = G_wp_;
                        
                    end
                    
                    [ L_ , ZStar_ ] =  ... 
                        AggregatesCalculation( G_wp_ , w_Zp_ , Z_wp_ , Pit , at , L_prev , obj.Model );
                    
                    Criterion = mean( abs(L_ - L_prev) );
                    SSConverged = ( Criterion < Tol );                    
                    SSiter = SSiter + 1;
                    if ( SSiter > MaxIt )
                        error('Maximum number of iterations exceeded in steady state loop')
                    end
                                        
                    L_prev = L_;
                    
                end
                
                obj.L = L_;
                obj.ZStar = ZStar_;
                obj.u = 100.*(obj.Model.FlexWage.L-L_)./obj.Model.FlexWage.L;
                obj.w_Zp = w_Zp_ ;             
                obj.Z_wp = Z_wp_ ;
                obj.V_wp = V_wp_ ;
                obj.G_wp = G_wp_ ;                              
                                                
                obj.Eta_s = obj.Model.Parameters.Eta;
                obj.Beta_s = obj.Model.Parameters.Beta;
                obj.Gamma_s = obj.Model.Parameters.Gamma; 
                obj.Sigma_s = obj.Model.Parameters.Sigma;
                obj.Lamda_s = obj.Model.Parameters.Lamda;
                obj.aBar_s = obj.Model.Parameters.aBar;
                obj.PiBar_s = obj.Model.Parameters.PiBar;
                                
                Solved_ = true;
                
            end
        end
            
            function Gw = G(obj,w)
               
                if obj.IsSolved 
                    Gw = ppvalcutoff(obj.G_wp,w,0,1);
                else
                	Gw = NaN ;
                end
                
            end
            
            function gw = g(obj,w)
               
                g_wp = ppder( obj.G_wp );
                
                if obj.IsSolved 
                    gw = ppvalcutoff(g_wp,w,0,0); 
                else
                	gw = NaN ;
                end
                
            end
            
            function hw = h(obj, dlnW )
            
               if ~( (size(dlnW,1)==1) & (size(dlnW,2)==1) )
               
                   error('Can only calculate wage change density for scalar' )
                   
               end
                
               if obj.IsSolved
                    
                    if ( dlnW == 0 )
                        error('This method can not be used to determine spike at zero')
                    end
                   
                    ZGrid = obj.Model.Numerics.ZGrid;
                    w_Z = polyval( obj.w_Zp , ZGrid );
                    g_wp = ppder( obj.G_wp );
                    g_wg = ppvalcutoff( g_wp , w_Z.*(1+obj.Pi).*(1+obj.a).*exp(-dlnW) , 0 , 0 );                
                    g_wp = polyfit( ZGrid , g_wg , obj.Model.Numerics.PolyOrder );
                    hw = ( 1 - obj.Model.Parameters.Lamda.*(dlnW < 0 ) ) .* ...
                          PolyFZInt( g_wp , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );

               else    
                    hw = NaN;
               end 
                
            end
            
            function Hw = H(obj, dlnW )           
            
               if ~( (size(dlnW,1)==1) & (size(dlnW,2)==1) )
               
                   error('Can only calculate wage change density for scalar' )
                   
               end
                
               if obj.IsSolved
                    
                    ZGrid = obj.Model.Numerics.ZGrid;
                    w_Z = polyval( obj.w_Zp , ZGrid );
                    G_w1g = 1-ppvalcutoff( obj.G_wp , w_Z.*(1+obj.Pi).*(1+obj.a).*exp(-dlnW) , 0 , 1 );
                    G_w2g = (1 - obj.Model.Parameters.Lamda.*(dlnW<=0)).*G_w1g;
                    G_w2p = polyfit( ZGrid , G_w2g , obj.Model.Numerics.PolyOrder );
                    Hw = PolyFZInt( G_w2p , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );

               else    
                    Hw = NaN;
               end
              
             end                 
            
        end
        
    end
    

