classdef TransitionPath < handle
    %TRANSITIONPATH Transition path of the model
    %
    %   Properties
    %   ----------
    %   Type        Object type 'steady state'
    %   Model       Model for which object is steady state
    %   L           Level of output/employment in the steady state
    %   ZStar       Adjusted labor wedge
    %   wMax        Maximum wage charged
    %   wMin        Minimum wage charged
    %   u           Transition path of the unemployment rate
    %   Pi          Transition path of the inflation rate
    %   a           Transition path of productivity growth
    %   r           Transition path of the real interest rate
    %   i           Transition path of the nominal interest rate
    %   w_Zp        Polynomial with the wage-setting function
    %   Z_wp        Polynomial with the inverse of the wage-setting
    %                   function
    %   V_wp        Polynomial with the value function approximation
    %   G_wp        Polynomial with the wage distribution approximation
    %   h0          Spike at zero nominal wage change
    %   h04Q        Spike at zero nominal wage change over 4 quarters
    %   L           Labor input (hours worked)
    %   IsSolved    Boolean that flags whether steady state values are 
    %               for the current model parameters or whether it needs
    %               to be solved for these parameters.   
    %
    %   Methods
    %   -------
    %   G           Distribution function of real wages
    %   g           Density function of real wages
    %   H           Distribution function of log nominal wage changes
    %   h           Density function of log nominal wage changes
    %   Solve       Solve for the steady state
    %   
    
    properties
        a
        D
    end
    
    properties ( SetAccess = private , Hidden = false )
        Type = 'transition path'
        Model
        L
        ZStar
        u        
        w_Zp
        Z_wp
        V_wp
        G_wp
        Pi
        r
        i
        h0
        h04Q        
    end
    
    properties ( SetAccess = private , Hidden = true )
        
        Eta_s
        Beta_s
        Gamma_s
        Sigma_s
        Lamda_s
        PiBar_s
        aBar_s
        PhiY_s
        PhiPi_s
        D_s
        T_s
        
    end
    
    properties ( Dependent = true , SetAccess = private )
       
        IsSolved
        
    end
    
    methods
        
        function obj = TransitionPath( MyParent )
            % Constructor that tracks what type of parent object has
            if ~strcmp(MyParent.Type,'model')
                error('Parent object needs to be a model object');
            end                     
                                    
            obj.Model = MyParent;            
            
            T = obj.Model.Numerics.T;
            
            obj.L = zeros(T+2,1);
            obj.ZStar = zeros(T+2,1);
            obj.u = zeros(T+2,1);
            obj.w_Zp = zeros(T+2,obj.Model.Numerics.PolyOrder+1);             
            obj.Z_wp = zeros(T+2,obj.Model.Numerics.PolyOrder+1);
            obj.V_wp = zeros(T+2,obj.Model.Numerics.PolyOrder+1);
            obj.G_wp = zeros(T+2,obj.Model.Numerics.PolyOrder+1);  
            obj.Pi = zeros(T+2,1);
            obj.a = ones(T+2,1).*obj.Model.Parameters.aBar;
            obj.D = zeros(T+2,1);
            obj.r = zeros(T+2,1);
            obj.i = zeros(T+2,1);
            obj.h0 = zeros(T+2,1);
            obj.h04Q = zeros(T+2,1);
               
            obj.T_s = T;
            obj.Eta_s = 0;
            obj.Beta_s = 0;
            obj.Gamma_s = 0; 
            obj.Sigma_s = 0;
            obj.Lamda_s = 0;
            obj.PiBar_s = 0;
            obj.aBar_s = 0;
            obj.PhiY_s = 0;
            obj.PhiPi_s = 0;
            obj.D_s = zeros( T+2 , 1 );
            
        end                               
        
        function obj = set.a(obj,val)
           T = obj.Model.Numerics.T;
           if ~ ( ( size(val,1) == (T+2) ) & ( size(val,2) == 1 ) )
              error('Path of productivity growth rate does not have the right length')
           end
           if ~ ( ( max(val) < 1 ) & ( min(val) > -1 ) ) 
              error('Are you sure you would like to set the trend productivity growth rate to this value?')
           end
           obj.a = val;
        end
        
        function obj = set.D(obj,val)
           T = obj.Model.Numerics.T;
           if ~ ( ( size(val,1) == (T+2) ) & ( size(val,2) == 1 ) )
              error('Path of discount rate shock does not have the right length')
           end
           if ~ ( ( max(val) < 1 ) & ( min(val) > -1 ) ) 
              error('Are you sure you would like to set the discount rate shock to this value?')
           end
           obj.D = val;
        end
        
        function IsSolved_ = get.IsSolved(obj)
           
            T = obj.Model.Numerics.T;
            
            if ( T == obj.T_s )
                IsSolved_ = ...
                    ( obj.Eta_s == obj.Model.Parameters.Eta ) & ...
                    ( obj.Beta_s == obj.Model.Parameters.Beta ) & ...
                    ( obj.Gamma_s == obj.Model.Parameters.Gamma ) & ... 
                    ( obj.Sigma_s == obj.Model.Parameters.Sigma ) & ...
                    ( obj.Lamda_s == obj.Model.Parameters.Lamda ) & ...
                    ( obj.PiBar_s == obj.Model.Parameters.PiBar ) & ...
                    ( obj.aBar_s == obj.Model.Parameters.aBar ) & ...
                    ( obj.PhiY_s == obj.Model.Parameters.PhiY ) & ...
                    ( obj.PhiPi_s == obj.Model.Parameters.PhiPi ) & ...
                    ( sum(obj.D == obj.D_s) == T+2  ) ;
            else
                IsSolved_ = false;
            end
                
        end     
        
        function Solved_ = Solve( obj )      
            
            if ~( obj.Model.SteadyState.IsSolved )
                error('You will first have to solve the steady state before you can solve the transition path');
            end
            
            T = obj.Model.Numerics.T;
            
            obj.L = ones(T+2,1).*obj.Model.SteadyState.L;
            obj.ZStar = ones(T+2,1).*obj.Model.SteadyState.ZStar;
            obj.u = ones(T+2,1).*obj.Model.SteadyState.u;
            obj.w_Zp = kron(obj.Model.SteadyState.w_Zp,ones(T+2,1));             
            obj.Z_wp = kron(obj.Model.SteadyState.Z_wp,ones(T+2,1));
            obj.V_wp = kron(obj.Model.SteadyState.V_wp,ones(T+2,1));
            obj.G_wp = obj.Model.SteadyState.G_wp;
            for t_ = 1:(T+1)
                obj.G_wp = [ obj.G_wp ; obj.Model.SteadyState.G_wp ];
            end
            obj.Pi = ones(T+2,1).*obj.Model.SteadyState.Pi;
            obj.r = ones(T+2,1).*obj.Model.SteadyState.r;
            obj.i = ones(T+2,1).*obj.Model.SteadyState.i;
            obj.h0 = ones(T+2,1).*obj.Model.SteadyState.h0;
            obj.h04Q = ones(T+2,1).*obj.Model.SteadyState.h04Q;
            
            if ~( obj.IsSolved )
               
                Tol = obj.Model.Numerics.TPTol;
                MaxIt = obj.Model.Numerics.MaxIt;
                T = obj.Model.Numerics.T;                                                                              
                
                msg = strcat('Model: ',obj.Model.Name,', Solving Transitional Path');            
                
                Criterion = NaN;
                TPConverged = false;
                TPiter = 0;
                while ( TPConverged == false )                  

                    L_prev = obj.L;
                    Pi_prev = obj.Pi;                
                                                                            
                    msg2 = strcat('Iteration:  ',num2str(TPiter),', criterion:  ', num2str(Criterion) );
                        
                    for t_ = (T+1):-1:2                       
                        
                        Pi_Up = 0;                                                    
                        [ ExcessDemand_Up ] = ADMinusAS( obj.V_wp(t_+1,:) , obj.G_wp(t_-1,:) , obj.L(t_+1,1) , Pi_Up , obj.Pi(t_+1,1) , obj.D(t_,1) , obj.a(t_,1) , obj.a(t_+1,1) , obj.Model );

                        msg3 = strcat('Backward iteration over AD and AS, t=', num2str(t_-1));
                        if obj.Model.Display == 1 

                            clc;
                            disp(msg);
                            disp(msg2);
                            disp(msg3);

                        end
                        
                        while ( ExcessDemand_Up > 0 );
                            
                            msg3 = strcat('Backward iteration over AD and AS, t=', num2str(t_-1),' Pi_Up=', num2str( Pi_Up ) ,' Excess demand=' , num2str( ExcessDemand_Up ) );
                            if obj.Model.Display > 1 
                           
                                clc;
                                disp(msg);
                                disp(msg2);
                                disp(msg3);

                            end
                            
                            Pi_Up = sqrt( 2.*(1+Pi_Up) ) - 1 ;
                            [ ExcessDemand_Up ] = ADMinusAS( obj.V_wp(t_+1,:) , obj.G_wp(t_-1,:) , obj.L(t_+1,1) , Pi_Up , obj.Pi(t_+1,1) , obj.D(t_,1) , obj.a(t_,1) , obj.a(t_+1,1) , obj.Model );                                                                
                        end

                        Pi_Low = 0;
                        [ ExcessDemand_Low ] = ADMinusAS( obj.V_wp(t_+1,:) , obj.G_wp(t_-1,:) , obj.L(t_+1,1) , Pi_Low , obj.Pi(t_+1,1) , obj.D(t_,1) , obj.a(t_,1) , obj.a(t_+1,1) , obj.Model );
                        while ( ExcessDemand_Low < 0 );
                            
                            msg3 = strcat('Backward iteration over AD and AS, t=', num2str(t_-1),' Pi_Low=', num2str( Pi_Low ) ,' Excess demand=' , num2str( ExcessDemand_Low ) );
                            if obj.Model.Display > 1 
                           
                                clc;
                                disp(msg);
                                disp(msg2);
                                disp(msg3);

                            end
                                                        
                            Pi_Low = sqrt( 0.5*(1+Pi_Low) ) - 1 ;
                            [ ExcessDemand_Low ] = ADMinusAS( obj.V_wp(t_+1,:) , obj.G_wp(t_-1,:) , obj.L(t_+1,1) , Pi_Low , obj.Pi(t_+1,1) , obj.D(t_,1) , obj.a(t_,1) , obj.a(t_+1,1) , obj.Model );                                                                
                        end

                        GSCriterion = ( abs( Pi_Up - Pi_Low ) < (obj.Model.Numerics.Tol./100) );
                        while ~GSCriterion
                            
                            msg3 = strcat('Backward iteration over AD and AS, t=', num2str(t_-1),' criterion=', num2str( abs( Pi_Up - Pi_Low ) ) );
                            if obj.Model.Display > 1 
                           
                                clc;
                                disp(msg);
                                disp(msg2);
                                disp(msg3);

                            end
                            
                            Pi_Mid = sqrt( (1+Pi_Up).*(1+Pi_Low) )-1;
                            [ ExcessDemand_Mid ] = ADMinusAS( obj.V_wp(t_+1,:) , obj.G_wp(t_-1,:) , obj.L(t_+1,1) , Pi_Mid , obj.Pi(t_+1,1) , obj.D(t_,1) , obj.a(t_,1) , obj.a(t_+1,1) , obj.Model );                                                                

                            if ( ExcessDemand_Mid > 0 )

                                Pi_Low = Pi_Mid;

                            else

                                Pi_Up = Pi_Mid;

                            end

                            GSCriterion = ( abs( Pi_Up - Pi_Low ) < (obj.Model.Numerics.Tol./10) );
                        end

                        obj.Pi(t_,1) = sqrt( (1+Pi_Up).*(1+Pi_Low ) ) - 1;
                        [ obj.L(t_,1) , obj.i(t_,1) , obj.r(t_,1) ] = ADCurve( obj.Pi(t_,1) , obj.L(t_+1,1) , obj.Pi(t_+1,1) , obj.a(t_+1,1) , obj.D(t_,1) , obj.Model );
                        [ obj.w_Zp(t_,:) , obj.Z_wp(t_,:) , obj.V_wp(t_+1,:) ] = ... 
                            ValueFunctionIteration( obj.V_wp(t_+1,:) , obj.L(t_,1) , obj.D(t_,1) , obj.Pi(t_+1,1) , obj.a(t_+1,1) , obj.Model );                            

                    end

                    for t_ = 2:(T+1)

                        msg3 = strcat('Forward iteration over state variable, t=', num2str(t_-1) );                       
                        
                        if obj.Model.Display > 0 
                           
                            clc;
                            disp(msg);
                            disp(msg2);
                            disp(msg3);
                            
                        end
                        
                        [ g_t_wp , obj.G_wp(t_) ] = WageDistributionIteration( obj.G_wp(t_-1) , obj.Z_wp(t_,:) , obj.w_Zp(t_,:) , obj.Pi(t_,1) , obj.a(t_,1) , obj.Model );

                    end                                                                                            
                    
                    u_Prev = obj.u;
                    obj.u = 100.*(obj.Model.FlexWage.L-obj.L)./obj.Model.FlexWage.L;
                    
                    Criterion = [ ( obj.u - u_Prev).^2 ] ;                                  
                    
                    Criterion = sqrt(mean( Criterion ));
                    TPConverged = ( Criterion < Tol );
                    TPiter = TPiter + 1;
                    
                    if ( TPiter > MaxIt )
                        error('Maximum number of iterations exceeded in real side path loop')
                    end                 
                    
                    if obj.Model.Display == 1 
                           
                        plot( [ obj.u u_Prev ] );
                        
                    end
                    
                end                                                                                                          
                
                for t_ = 2:(T+1)
                   
                    ZGrid = obj.Model.Numerics.ZGrid;
                    w_Z = polyval( obj.w_Zp(t_,:) , ZGrid );
                    G_wg = ppvalcutoff( obj.G_wp(t_) , w_Z.*(1+obj.Pi(t_)).*(1+obj.a(t_)) , 0 , 1 );                
                    OneMinusG_wg = 1 - G_wg;
                    OneMinusG_wp = polyfit( ZGrid , OneMinusG_wg  , obj.Model.Numerics.PolyOrder );
                    obj.h0(t_,1) = obj.Model.Parameters.Lamda .* ...
                        PolyFZInt( OneMinusG_wp , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );
                    
                end
                
                for t_ = 2:(T+1)
                   
                    if t_ < 5 
                        
                        if t_ == 1
                          
                          Z_wp_ = [ obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ];
                          pi_   =  [ obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ] ;
                          a_    =  [ obj.a(1,1) ; ...
                                     obj.a(1,1) ; ...
                                     obj.a(1,1) ; ...
                                     obj.a(1,1) ] ;           
                        elseif t_ == 2
                            
                          Z_wp_ = [ obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(2,:) ];
                          pi_   =  [ obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ; ...
                                     obj.Pi(2,1) ] ;
                          a_   =   [ obj.a(1,1) ; ...
                                     obj.a(1,1) ; ...
                                     obj.a(1,1) ; ...
                                     obj.a(2,1) ] ;
                                     
                        elseif t_ == 3
                            
                          Z_wp_ = [ obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(2,:) ; ...
                                    obj.Z_wp(3,:) ];
                          pi_   =  [ obj.Pi(1,1) ; ...
                                     obj.Pi(1,1) ; ...
                                     obj.Pi(2,1) ; ...
                                     obj.Pi(3,1) ] ;
                          a_   =   [ obj.a(1,1) ; ...
                                     obj.a(1,1) ; ...
                                     obj.a(2,1) ; ...
                                     obj.a(3,1) ] ;           
                        elseif t_ == 4
                            
                          Z_wp_ = [ obj.Z_wp(1,:) ; ...
                                    obj.Z_wp(2,:) ; ...
                                    obj.Z_wp(3,:) ; ...
                                    obj.Z_wp(4,:) ];
                          pi_   =  [ obj.Pi(1,1) ; ...
                                     obj.Pi(2,1) ; ...
                                     obj.Pi(3,1) ; ...
                                     obj.Pi(4,1) ] ;
                          a_   =   [ obj.a(1,1) ; ...
                                     obj.a(2,1) ; ...
                                     obj.a(3,1) ; ...
                                     obj.a(4,1) ] ;           
                        else
                        
                        end
                            
                        obj.h04Q(t_,1) = H04QCalculation( obj.G_wp(1) , Z_wp_ , pi_ , a_ , obj.Model );
                        
                    else
                        obj.h04Q(t_,1) = H04QCalculation( obj.G_wp(t_-4) , obj.Z_wp((t_-3):t_,:) , obj.Pi((t_-3):t_,1) , obj.a((t_-3):t_,1) , obj.Model );
                    end
                    
                end
                
                obj.Eta_s = obj.Model.Parameters.Eta;
                obj.Beta_s = obj.Model.Parameters.Beta;
                obj.Gamma_s = obj.Model.Parameters.Gamma; 
                obj.Sigma_s = obj.Model.Parameters.Sigma;
                obj.Lamda_s = obj.Model.Parameters.Lamda;
                obj.PiBar_s = obj.Model.Parameters.PiBar;
                obj.aBar_s = obj.Model.Parameters.aBar;
                obj.PhiPi_s = obj.Model.Parameters.PhiPi;
                obj.PhiY_s = obj.Model.Parameters.PhiY;
                obj.D_s = obj.D;
                obj.T_s = T;
                                
                Solved_ = true;
                
            end
        end
            
            function Gw = G(obj,w , t)
               
                if obj.IsSolved 
                    Gw = ppvalcutoff(obj.G_wp(t+1),w,0,1);
                else
                	Gw = NaN ;
                end
                
            end
            
            function gw = g(obj,w, t)
               
                g_wp = ppder( obj.G_wp(t+1) );
                
                if obj.IsSolved 
                    gw = ppvalcutoff(g_wp,w,0,0); 
                else
                	gw = NaN ;
                end
                
            end
            
            function hw = h(obj, dlnW , t )
            
              if ~( (size(dlnW,1)==1) & (size(dlnW,2)==1) )
               
                   error('Can only calculate wage change density for scalar' )
                   
               end
                
               if obj.IsSolved
                    
                    if ( dlnW == 0 )
                        error('This method can not be used to determine spike at zero')
                    end
                   
                    ZGrid = obj.Model.Numerics.ZGrid;
                    w_Z = polyval( obj.w_Zp(t+1,:) , ZGrid );
                    g_wp = ppder( obj.G_wp(t+1) );
                    g_wg = ppvalcutoff( g_wp , w_Z.*(1+obj.Pi(t+1,1)).*(1+obj.a(t+1,1)).*exp(-dlnW) , 0 , 0 );                
                    g_wp = polyfit( ZGrid , g_wg , obj.Model.Numerics.PolyOrder );
                    hw = ( 1 - obj.Model.Parameters.Lamda.*(dlnW < 0 ) ) .* ...
                          PolyFZInt( g_wp , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );

               else    
                    hw = NaN;
               end 
                
            end
            
            function Hw = H(obj, dlnW , t )           
            
               if ~( (size(dlnW,1)==1) & (size(dlnW,2)==1) )
               
                   error('Can only calculate wage change density for scalar' )
                   
               end
                
               if obj.IsSolved
                    
                    ZGrid = obj.Model.Numerics.ZGrid;
                    w_Z = polyval( obj.w_Zp(t+1,:) , ZGrid );
                    G_w1g = 1-ppvalcutoff( obj.G_wp(t+1) , w_Z.*(1+obj.Pi(t+1,1)).*(1+obj.a(t+1,1)).*exp(-dlnW) , 0 , 1 );
                    G_w2g = (1 - obj.Model.Parameters.Lamda.*(dlnW<=0)).*G_w1g;
                    G_w2p = polyfit( ZGrid , G_w2g , obj.Model.Numerics.PolyOrder );
                    Hw = PolyFZInt( G_w2p , 0 , obj.Model.Numerics.ZLow , obj.Model.Numerics.ZUp , obj.Model );

               else    
                    Hw = NaN;
               end
              
            end       
             
            function D_ = ShockPath(obj,d,l,tstar)
                 
               obj.D = DCalculation( d , l , tstar , obj.Model );
               D_ = obj.D;
               
            end
                
            function D_ = ShockPathAR1(obj,d,rho)
               
               GridPoints = obj.Model.Numerics.T;
               tg = (cumsum(ones(GridPoints,1))-1);
                
               obj.D = [ 0 ; d.*(rho.^tg) ; 0 ];
               D_ = obj.D;
               
            end
            
            function D_ = ShockPathAR1Delay(obj,d,rho,l)
               
               GridPoints = obj.Model.Numerics.T;
               tl = (1:l)'./l;
               tl = [ tl ; zeros(GridPoints-l,1) ];
               tl = d.*tl;
               tg = (cumsum(ones(GridPoints-l,1)));
               tg = d.*( rho.^tg );
               tg = [ zeros(l,1) ; tg ];
                
               obj.D = [ 0 ; (tl + tg) ; 0 ];
               D_ = obj.D;
               
            end
            
        end
        
    end
    

